pub mod aws_s3;
pub mod nats;
pub mod grpc {
    use std::sync::Arc;

    use anyhow::Context;
    use tokio::sync::Mutex;
    use tonic::transport::Channel;

    use crate::grpc::gen::flux_releaser_client::FluxReleaserClient;

    pub type FluxReleaserGrpcClient = Arc<Mutex<FluxReleaserClient<Channel>>>;

    pub async fn new_client(registry: impl Into<String>) -> anyhow::Result<FluxReleaserGrpcClient> {
        let registry = registry.into();

        let client = FluxReleaserClient::connect(registry)
            .await
            .context("failed to connect to flux_releaser registry")?;

        Ok(Arc::new(Mutex::new(client)))
    }
}
